@@ -62,12 +62,12 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("period_add") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length=6*MY_CHARSET_BIN_MB_MAXLEN;
     return FALSE;
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_period_add>(thd, this); }
 };
 
@@ -84,13 +84,13 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("period_diff") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=6*MY_CHARSET_BIN_MB_MAXLEN;
     return FALSE;
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_period_diff>(thd, this); }
 };
 
@@ -105,7 +105,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("to_days") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0; 
     max_length=6*MY_CHARSET_BIN_MB_MAXLEN;
@@ -120,7 +120,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_to_days>(thd, this); }
 };
 
@@ -137,7 +137,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("to_seconds") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0; 
     fix_char_length(12);
@@ -153,7 +153,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_to_seconds>(thd, this); }
 };
 
@@ -168,7 +168,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("dayofmonth") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0; 
     max_length=2*MY_CHARSET_BIN_MB_MAXLEN;
@@ -181,7 +181,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_dayofmonth>(thd, this); }
 };
 
@@ -197,7 +197,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("month") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals= 0;
     fix_char_length(2);
@@ -210,7 +210,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_month>(thd, this); }
 };
 
@@ -226,7 +226,7 @@ public:
     return name;
   }
   String *val_str(String *str) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool check_partition_func_processor(void *int_arg) override {return TRUE;}
   bool check_valid_arguments_processor(void *int_arg) override
   {
@@ -236,7 +236,7 @@ public:
   {
     return mark_unsupported_function(func_name(), "()", arg, VCOL_SESSION_FUNC);
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_monthname>(thd, this); }
 };
 
@@ -251,7 +251,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("dayofyear") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals= 0;
     fix_char_length(3);
@@ -264,7 +264,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_dayofyear>(thd, this); }
 };
 
@@ -279,7 +279,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("hour") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=2*MY_CHARSET_BIN_MB_MAXLEN;
@@ -292,7 +292,7 @@ public:
   {
     return !has_time_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_hour>(thd, this); }
 };
 
@@ -307,7 +307,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("minute") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=2*MY_CHARSET_BIN_MB_MAXLEN;
@@ -320,7 +320,7 @@ public:
   {
     return !has_time_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_minute>(thd, this); }
 };
 
@@ -335,7 +335,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("quarter") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
      decimals=0;
      max_length=1*MY_CHARSET_BIN_MB_MAXLEN;
@@ -348,7 +348,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_quarter>(thd, this); }
 };
 
@@ -363,7 +363,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("second") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=2*MY_CHARSET_BIN_MB_MAXLEN;
@@ -376,7 +376,7 @@ public:
   {
     return !has_time_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_second>(thd, this); }
 };
 
@@ -397,7 +397,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("week") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=2*MY_CHARSET_BIN_MB_MAXLEN;
@@ -414,7 +414,7 @@ public:
   {
     return arg_count == 2;
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_week>(thd, this); }
 };
 
@@ -434,7 +434,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("yearweek") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=6*MY_CHARSET_BIN_MB_MAXLEN;
@@ -447,7 +447,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_yearweek>(thd, this); }
 };
 
@@ -464,7 +464,7 @@ public:
   }
   enum_monotonicity_info get_monotonicity_info() const override;
   longlong val_int_endpoint(bool left_endp, bool *incl_endp) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     max_length=4*MY_CHARSET_BIN_MB_MAXLEN;
@@ -477,7 +477,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_year>(thd, this); }
 };
 
@@ -499,7 +499,7 @@ public:
   {
     return type_handler()->Item_get_date_with_warn(thd, this, ltime, fuzzydate);
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals= 0;
     fix_char_length(1);
@@ -512,7 +512,7 @@ public:
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_weekday>(thd, this); }
 };
 
@@ -529,7 +529,7 @@ class Item_func_dayname :public Item_str_func
   String *val_str(String *str) override;
   const Type_handler *type_handler() const override
   { return &type_handler_varchar; }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool check_partition_func_processor(void *int_arg) override {return TRUE;}
   bool check_vcol_func_processor(void *arg) override
   {
@@ -539,7 +539,7 @@ class Item_func_dayname :public Item_str_func
   {
     return !has_date_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_dayname>(thd, this); }
 };
 
@@ -560,9 +560,9 @@ public:
     else
       set_handler(type_handler_long_or_longlong());
   }
-  double real_op() override { DBUG_ASSERT(0); return 0; }
-  String *str_op(String *str) override { DBUG_ASSERT(0); return 0; }
-  bool date_op(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override
+  double real_op() { DBUG_ASSERT(0); return 0; }
+  String *str_op(String *str) { DBUG_ASSERT(0); return 0; }
+  bool date_op(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate)
   {
     DBUG_ASSERT(0);
     return true;
@@ -600,15 +600,15 @@ public:
       return FALSE;
     return mark_unsupported_function(func_name(), "()", arg, VCOL_TIME_FUNC);
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     fix_length_and_dec_generic(arg_count ?
-                               args[0]->datetime_precision(current_thd) : 0);
+                               args[0]->datetime_precision(thd) : 0);
     return FALSE;
   }
   longlong int_op() override;
   my_decimal *decimal_op(my_decimal* buf) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_unix_timestamp>(thd, this); }
 };
 
@@ -629,14 +629,14 @@ public:
   {
     return !has_time_args();
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    fix_length_and_dec_generic(args[0]->time_precision(current_thd));
+    fix_length_and_dec_generic(args[0]->time_precision(thd));
     return FALSE;
   }
   longlong int_op() override;
   my_decimal *decimal_op(my_decimal* buf) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_time_to_sec>(thd, this); }
 };
 
@@ -657,7 +657,7 @@ public:
   { return Date(this).to_string(to); }
   my_decimal *val_decimal(my_decimal *to) override
   { return Date(this).to_decimal(to); }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     fix_attributes_date();
     set_maybe_null(arg_count > 0);
@@ -717,7 +717,7 @@ public:
   Item_func_curtime(THD *thd, uint dec): Item_timefunc(thd), last_query_id(0)
   { decimals= dec; }
   bool fix_fields(THD *, Item **) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   { fix_attributes_time(decimals); return FALSE; }
   bool get_date(THD *thd, MYSQL_TIME *res, date_mode_t fuzzydate) override;
   /* 
@@ -744,7 +744,7 @@ public:
     return name;
   }
   void store_now_in_TIME(THD *thd, MYSQL_TIME *now_time) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_curtime_local>(thd, this); }
 };
 
@@ -759,7 +759,7 @@ public:
     return name;
   }
   void store_now_in_TIME(THD *thd, MYSQL_TIME *now_time) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_curtime_utc>(thd, this); }
 };
 
@@ -791,7 +791,7 @@ public:
     return name;
   }
   void store_now_in_TIME(THD *thd, MYSQL_TIME *now_time) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_curdate_local>(thd, this); }
 };
 
@@ -806,7 +806,7 @@ public:
     return name;
   }
   void store_now_in_TIME(THD* thd, MYSQL_TIME *now_time) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_curdate_utc>(thd, this); }
 };
 
@@ -821,7 +821,7 @@ public:
   Item_func_now(THD *thd, uint dec): Item_datetimefunc(thd), last_query_id(0)
   { decimals= dec; }
   bool fix_fields(THD *, Item **) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   { fix_attributes_datetime(decimals); return FALSE;}
   bool get_date(THD *thd, MYSQL_TIME *res, date_mode_t fuzzydate) override;
   virtual void store_now_in_TIME(THD *thd, MYSQL_TIME *now_time)=0;
@@ -849,7 +849,7 @@ public:
   int save_in_field(Field *field, bool no_conversions) override;
   void store_now_in_TIME(THD *thd, MYSQL_TIME *now_time) override;
   enum Functype functype() const override { return NOW_FUNC; }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_now_local>(thd, this); }
 };
 
@@ -870,7 +870,7 @@ public:
     return mark_unsupported_function(func_name(), "()", arg,
                                      VCOL_TIME_FUNC | VCOL_NON_DETERMINISTIC);
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_now_utc>(thd, this); }
 };
 
@@ -898,7 +898,7 @@ public:
                                      VCOL_TIME_FUNC | VCOL_NON_DETERMINISTIC);
   }
   enum Functype functype() const override { return SYSDATE_FUNC; }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_sysdate_local>(thd, this); }
 };
 
@@ -921,7 +921,7 @@ public:
   {
     return has_date_args() || has_time_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_from_days>(thd, this); }
 };
 
@@ -949,7 +949,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("date_format") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   uint format_length(const String *format);
   bool eq(const Item *item, bool binary_cmp) const override;
   bool check_vcol_func_processor(void *arg) override
@@ -958,7 +958,7 @@ public:
       return false;
     return mark_unsupported_function(func_name(), "()", arg, VCOL_SESSION_FUNC);
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_date_format>(thd, this); }
 };
 
@@ -973,7 +973,7 @@ public:
     return name;
   }
   bool check_vcol_func_processor(void *arg) override { return false; }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_time_format>(thd, this); }
 };
 
@@ -1017,7 +1017,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("to_char") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool parse_format_string(const String *format, uint *fmt_len);
 
   bool check_vcol_func_processor(void *arg) override
@@ -1027,7 +1027,7 @@ public:
     return mark_unsupported_function(func_name(), "()", arg, VCOL_SESSION_FUNC);
   }
 
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_tochar>(thd, this); }
 };
 
@@ -1044,13 +1044,13 @@ class Item_func_from_unixtime :public Item_datetimefunc
     static LEX_CSTRING name= {STRING_WITH_LEN("from_unixtime") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool get_date(THD *thd, MYSQL_TIME *res, date_mode_t fuzzydate) override;
   bool check_vcol_func_processor(void *arg) override
   {
     return mark_unsupported_function(func_name(), "()", arg, VCOL_SESSION_FUNC);
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_from_unixtime>(thd, this); }
 };
 
@@ -1092,15 +1092,15 @@ class Item_func_convert_tz :public Item_datetimefunc
     static LEX_CSTRING name= {STRING_WITH_LEN("convert_tz") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    fix_attributes_datetime(args[0]->datetime_precision(current_thd));
+    fix_attributes_datetime(args[0]->datetime_precision(thd));
     set_maybe_null();
     return FALSE;
   }
   bool get_date(THD *thd, MYSQL_TIME *res, date_mode_t fuzzydate) override;
   void cleanup() override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_convert_tz>(thd, this); }
 };
 
@@ -1112,7 +1112,7 @@ class Item_func_sec_to_time :public Item_timefunc
 public:
   Item_func_sec_to_time(THD *thd, Item *item): Item_timefunc(thd, item) {}
   bool get_date(THD *thd, MYSQL_TIME *res, date_mode_t fuzzydate) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     fix_attributes_time(args[0]->decimals);
     set_maybe_null();
@@ -1123,7 +1123,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("sec_to_time") };
     return name;
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_sec_to_time>(thd, this); }
 };
 
@@ -1142,12 +1142,12 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("date_add_interval") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool eq(const Item *item, bool binary_cmp) const override;
   void print(String *str, enum_query_type query_type) override;
   enum precedence precedence() const override { return INTERVAL_PRECEDENCE; }
   bool need_parentheses_in_default() override { return true; }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_date_add_interval>(thd, this); }
 };
 
@@ -1213,7 +1213,7 @@ class Item_extract :public Item_int_func,
     return name;
   }
   bool check_arguments() const override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool eq(const Item *item, bool binary_cmp) const override;
   void print(String *str, enum_query_type query_type) override;
   bool check_partition_func_processor(void *int_arg) override {return FALSE;}
@@ -1259,7 +1259,7 @@ class Item_extract :public Item_int_func,
     }
     return true;
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_extract>(thd, this); }
 };
 
@@ -1300,13 +1300,13 @@ public:
   void fix_length_and_dec_numeric();
   void fix_length_and_dec_str();
   void fix_length_and_dec_native_to_binary(uint32 octet_length);
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     return args[0]->type_handler()->Item_char_typecast_fix_length_and_dec(this);
   }
   void print(String *str, enum_query_type query_type) override;
   bool need_parentheses_in_default() override { return true; }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_char_typecast>(thd, this); }
 };
 
@@ -1320,7 +1320,7 @@ public:
                        &my_charset_latin1),
     m_fsp(fsp)
   { }
-  String *val_str(String *to) override
+  String *val_str(String *to)
   {
     Interval_DDhhmmssff it(current_thd, args[0], m_fsp);
     null_value= !it.is_valid_interval_DDhhmmssff();
@@ -1343,11 +1343,11 @@ public:
     print_cast_temporal(str, query_type);
   }
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     return args[0]->type_handler()->Item_date_typecast_fix_length_and_dec(this);
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_date_typecast>(thd, this); }
 };
 
@@ -1367,13 +1367,13 @@ public:
     print_cast_temporal(str, query_type);
   }
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     return args[0]->type_handler()->
            Item_time_typecast_fix_length_and_dec(this);
   }
   Sql_mode_dependency value_depends_on_sql_mode() const override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_time_typecast>(thd, this); }
 };
 
@@ -1393,13 +1393,13 @@ public:
     print_cast_temporal(str, query_type);
   }
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     return args[0]->type_handler()->
            Item_datetime_typecast_fix_length_and_dec(this);
   }
   Sql_mode_dependency value_depends_on_sql_mode() const override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_datetime_typecast>(thd, this); }
 };
 
@@ -1417,7 +1417,7 @@ public:
     return name;
   }
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_makedate>(thd, this); }
 };
 
@@ -1438,9 +1438,8 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("timestamp") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    THD *thd= current_thd;
     uint dec0= args[0]->datetime_precision(thd);
     uint dec1= Interval_DDhhmmssff::fsp(thd, args[1]);
     fix_attributes_datetime(MY_MAX(dec0, dec1));
@@ -1459,7 +1458,7 @@ public:
     return (null_value= Sec6_add(dt.get_mysql_time(), it.get_mysql_time(), 1).
                            to_datetime(ltime));
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_timestamp>(thd, this); }
 };
 
@@ -1484,14 +1483,14 @@ public:
   Item_func_add_time(THD *thd, Item *a, Item *b, bool neg_arg)
    :Item_handled_func(thd, a, b), sign(neg_arg ? -1 : 1)
   { }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING addtime= { STRING_WITH_LEN("addtime") };
     static LEX_CSTRING subtime= { STRING_WITH_LEN("subtime") };
     return sign > 0 ? addtime : subtime;
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_add_time>(thd, this); }
 };
 
@@ -1507,9 +1506,8 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("timediff") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    THD *thd= current_thd;
     uint dec= MY_MAX(args[0]->time_precision(thd),
                      args[1]->time_precision(thd));
     fix_attributes_time(dec);
@@ -1517,7 +1515,7 @@ public:
     return FALSE;
   }
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_timediff>(thd, this); }
 };
 
@@ -1532,7 +1530,7 @@ public:
   Item_func_maketime(THD *thd, Item *a, Item *b, Item *c):
     Item_timefunc(thd, a, b, c)
   {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     fix_attributes_time(args[2]->decimals);
     set_maybe_null();
@@ -1544,7 +1542,7 @@ public:
     return name;
   }
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_maketime>(thd, this); }
 };
 
@@ -1559,7 +1557,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("microsecond") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     set_maybe_null();
@@ -1572,7 +1570,7 @@ public:
   {
     return !has_time_args();
   }
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_microsecond>(thd, this); }
 };
 
@@ -1594,14 +1592,14 @@ public:
     return name;
   }
   longlong val_int() override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals=0;
     set_maybe_null();
     return FALSE;
   }
   void print(String *str, enum_query_type query_type) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_timestamp_diff>(thd, this); }
 };
 
@@ -1624,7 +1622,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("get_format") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     decimals=0;
@@ -1632,7 +1630,7 @@ public:
     return FALSE;
   }
   void print(String *str, enum_query_type query_type) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_get_format>(thd, this); }
 };
 
@@ -1655,8 +1653,8 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("str_to_date") };
     return name;
   }
-  bool fix_length_and_dec() override;
-  Item *do_get_copy(THD *thd) const override
+  bool fix_length_and_dec(THD *thd) override;
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_str_to_date>(thd, this); }
 };
 
@@ -1673,7 +1671,7 @@ public:
     return name;
   }
   bool get_date(THD *thd, MYSQL_TIME *res, date_mode_t fuzzydate) override;
-  Item *do_get_copy(THD *thd) const override
+  Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_last_day>(thd, this); }
 };
 
@@ -1718,7 +1716,7 @@ class Func_handler_date_add_interval_datetime:
         public Func_handler_date_add_interval
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     uint dec= MY_MAX(item->arguments()[0]->datetime_precision(current_thd),
                      interval_dec(item->arguments()[1], int_type(item)));
@@ -1726,7 +1724,7 @@ public:
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     Datetime::Options opt(TIME_CONV_NONE, thd);
     Datetime dt(thd, item->arguments()[0], opt);
@@ -1745,7 +1743,7 @@ class Func_handler_date_add_interval_datetime_arg0_time:
 {
 public:
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override;
+                MYSQL_TIME *to, date_mode_t fuzzy) const;
 };
 
 
@@ -1755,7 +1753,7 @@ class Func_handler_date_add_interval_date:
 {
 public:
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     /*
       The first argument is known to be of the DATE data type (not DATETIME).
@@ -1777,7 +1775,7 @@ class Func_handler_date_add_interval_time:
         public Func_handler_date_add_interval
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     uint dec= MY_MAX(item->arguments()[0]->time_precision(current_thd),
                      interval_dec(item->arguments()[1], int_type(item)));
@@ -1785,7 +1783,7 @@ public:
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     Time t(thd, item->arguments()[0]);
     if (!t.is_valid_time())
@@ -1802,7 +1800,7 @@ class Func_handler_date_add_interval_string:
         public Func_handler_date_add_interval
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     uint dec= MY_MAX(item->arguments()[0]->datetime_precision(current_thd),
                      interval_dec(item->arguments()[1], int_type(item)));
@@ -1814,7 +1812,7 @@ public:
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     if (item->arguments()[0]->
           get_date(thd, to, Datetime::Options(TIME_CONV_NONE, thd)) ||
@@ -1843,7 +1841,7 @@ public:
   Func_handler_add_time_datetime(int sign)
    :Func_handler_sign(sign)
   { }
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     THD *thd= current_thd;
     uint dec0= item->arguments()[0]->datetime_precision(thd);
@@ -1852,7 +1850,7 @@ public:
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     DBUG_ASSERT(item->fixed());
     Datetime::Options opt(TIME_CONV_NONE, thd);
@@ -1877,7 +1875,7 @@ public:
   Func_handler_add_time_time(int sign)
    :Func_handler_sign(sign)
   { }
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     THD *thd= current_thd;
     uint dec0= item->arguments()[0]->time_precision(thd);
@@ -1886,7 +1884,7 @@ public:
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     DBUG_ASSERT(item->fixed());
     Time t(thd, item->arguments()[0]);
@@ -1910,7 +1908,7 @@ public:
   Func_handler_add_time_string(int sign)
    :Func_handler_sign(sign)
   { }
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     uint dec0= item->arguments()[0]->decimals;
     uint dec1= Interval_DDhhmmssff::fsp(current_thd, item->arguments()[1]);
@@ -1923,7 +1921,7 @@ public:
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     DBUG_ASSERT(item->fixed());
     // Detect a proper timestamp type based on the argument values
@@ -1947,13 +1945,13 @@ class Func_handler_str_to_date_datetime_sec:
         public Item_handled_func::Handler_datetime
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     item->fix_attributes_datetime(0);
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     return static_cast<Item_func_str_to_date*>(item)->
              get_date_common(thd, to, fuzzy, MYSQL_TIMESTAMP_DATETIME);
@@ -1965,13 +1963,13 @@ class Func_handler_str_to_date_datetime_usec:
         public Item_handled_func::Handler_datetime
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     item->fix_attributes_datetime(TIME_SECOND_PART_DIGITS);
     return false;
   }
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     return static_cast<Item_func_str_to_date*>(item)->
              get_date_common(thd, to, fuzzy, MYSQL_TIMESTAMP_DATETIME);
@@ -1983,7 +1981,7 @@ class Func_handler_str_to_date_date: public Item_handled_func::Handler_date
 {
 public:
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     return static_cast<Item_func_str_to_date*>(item)->
              get_date_common(thd, to, fuzzy, MYSQL_TIMESTAMP_DATE);
@@ -1995,7 +1993,7 @@ class Func_handler_str_to_date_time: public Item_handled_func::Handler_time
 {
 public:
   bool get_date(THD *thd, Item_handled_func *item,
-                MYSQL_TIME *to, date_mode_t fuzzy) const override
+                MYSQL_TIME *to, date_mode_t fuzzy) const
   {
     if (static_cast<Item_func_str_to_date*>(item)->
          get_date_common(thd, to, fuzzy, MYSQL_TIMESTAMP_TIME))
@@ -2018,7 +2016,7 @@ public:
 class Func_handler_str_to_date_time_sec: public Func_handler_str_to_date_time
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     item->fix_attributes_time(0);
     return false;
@@ -2029,7 +2027,7 @@ public:
 class Func_handler_str_to_date_time_usec: public Func_handler_str_to_date_time
 {
 public:
-  bool fix_length_and_dec(Item_handled_func *item) const override
+  bool fix_length_and_dec(Item_handled_func *item) const
   {
     item->fix_attributes_time(TIME_SECOND_PART_DIGITS);
     return false;
