@@ -164,6 +164,7 @@ TABLE_LIST *find_table_in_list(TABLE_LIST *table,
                                const LEX_CSTRING *db_name,
                                const LEX_CSTRING *table_name);
 int close_thread_tables(THD *thd);
+int close_thread_tables_for_query(THD *thd);
 void switch_to_nullable_trigger_fields(List<Item> &items, TABLE *);
 void switch_defaults_to_nullable_trigger_fields(TABLE *table);
 bool fill_record_n_invoke_before_triggers(THD *thd, TABLE *table,
@@ -314,8 +315,7 @@ bool flush_tables(THD *thd, flush_tables_type flag);
 void close_all_tables_for_name(THD *thd, TABLE_SHARE *share,
                                ha_extra_function extra,
                                TABLE *skip_table);
-OPEN_TABLE_LIST *list_open_tables(THD *thd, const LEX_CSTRING &db,
-                                  const char *wild);
+OPEN_TABLE_LIST *list_open_tables(THD *thd, const char *db, const char *wild);
 bool tdc_open_view(THD *thd, TABLE_LIST *table_list, uint flags);
 
 TABLE *find_table_for_mdl_upgrade(THD *thd, const char *db,
@@ -424,13 +424,13 @@ public:
 class DML_prelocking_strategy : public Prelocking_strategy
 {
 public:
-  bool handle_routine(THD *thd, Query_tables_list *prelocking_ctx,
-                      Sroutine_hash_entry *rt, sp_head *sp,
-                      bool *need_prelocking) override;
-  bool handle_table(THD *thd, Query_tables_list *prelocking_ctx,
-                    TABLE_LIST *table_list, bool *need_prelocking) override;
-  bool handle_view(THD *thd, Query_tables_list *prelocking_ctx,
-                   TABLE_LIST *table_list, bool *need_prelocking) override;
+  virtual bool handle_routine(THD *thd, Query_tables_list *prelocking_ctx,
+                              Sroutine_hash_entry *rt, sp_head *sp,
+                              bool *need_prelocking);
+  virtual bool handle_table(THD *thd, Query_tables_list *prelocking_ctx,
+                            TABLE_LIST *table_list, bool *need_prelocking);
+  virtual bool handle_view(THD *thd, Query_tables_list *prelocking_ctx,
+                           TABLE_LIST *table_list, bool *need_prelocking);
 };
 
 
@@ -441,8 +441,8 @@ public:
 
 class Lock_tables_prelocking_strategy : public DML_prelocking_strategy
 {
-  bool handle_table(THD *thd, Query_tables_list *prelocking_ctx,
-                    TABLE_LIST *table_list, bool *need_prelocking) override;
+  virtual bool handle_table(THD *thd, Query_tables_list *prelocking_ctx,
+                            TABLE_LIST *table_list, bool *need_prelocking);
 };
 
 
@@ -457,13 +457,13 @@ class Lock_tables_prelocking_strategy : public DML_prelocking_strategy
 class Alter_table_prelocking_strategy : public Prelocking_strategy
 {
 public:
-  bool handle_routine(THD *thd, Query_tables_list *prelocking_ctx,
-                      Sroutine_hash_entry *rt, sp_head *sp,
-                      bool *need_prelocking) override;
-  bool handle_table(THD *thd, Query_tables_list *prelocking_ctx,
-                    TABLE_LIST *table_list, bool *need_prelocking) override;
-  bool handle_view(THD *thd, Query_tables_list *prelocking_ctx,
-                   TABLE_LIST *table_list, bool *need_prelocking) override;
+  virtual bool handle_routine(THD *thd, Query_tables_list *prelocking_ctx,
+                              Sroutine_hash_entry *rt, sp_head *sp,
+                              bool *need_prelocking);
+  virtual bool handle_table(THD *thd, Query_tables_list *prelocking_ctx,
+                            TABLE_LIST *table_list, bool *need_prelocking);
+  virtual bool handle_view(THD *thd, Query_tables_list *prelocking_ctx,
+                           TABLE_LIST *table_list, bool *need_prelocking);
 };
 
 
@@ -537,7 +537,8 @@ public:
     OT_BACKOFF_AND_RETRY,
     OT_REOPEN_TABLES,
     OT_DISCOVER,
-    OT_REPAIR
+    OT_REPAIR,
+    OT_ADD_HISTORY_PARTITION
   };
   Open_table_context(THD *thd, uint flags);
 
@@ -564,6 +565,11 @@ public:
     return m_timeout;
   }
 
+  enum_open_table_action get_action() const
+  {
+    return m_action;
+  }
+
   uint get_flags() const { return m_flags; }
 
   /**
@@ -610,6 +616,9 @@ private:
     protection against global read lock.
   */
   mdl_bitmap_t m_has_protection_against_grl;
+
+public:
+  uint vers_create_count;
 };
 
 
@@ -653,7 +662,7 @@ public:
                         const char* sqlstate,
                         Sql_condition::enum_warning_level *level,
                         const char* msg,
-                        Sql_condition ** cond_hdl) override;
+                        Sql_condition ** cond_hdl);
 
   /**
     Returns TRUE if one or more ER_NO_SUCH_TABLE errors have been
